"""
enterprython - tests
"""

import configparser
import unittest
from abc import ABC, abstractmethod
from typing import NamedTuple, List

from ._inject import assemble, component, factory, value
from ._inject import set_values_from_config, add_values, set_values

__author__ = "Tobias Hermann"
__copyright__ = "Copyright 2018, Tobias Hermann"
__email__ = "editgym@gmail.com"
__license__ = "MIT"


class ServiceInterface(ABC):  # pylint: disable=too-few-public-methods
    """Defines interface of a service that can greet."""

    @abstractmethod
    def greet(self, name: str) -> str:
        """Shall return greeting message."""


@component()  # pylint: disable=too-few-public-methods
class Service(ServiceInterface):
    """Example service"""

    def greet(self, name: str) -> str:
        """Returns greeting message."""
        return f'Hello, {name}!'


@component(singleton=False)  # pylint: disable=too-few-public-methods
class ServiceNonSingleton:
    """Example service"""


class WithValue:  # pylint: disable=too-few-public-methods
    """Example class using a configuration value"""

    def __init__(self) -> None:
        """Reads a value from the configuration."""
        self._value: int = value(int, 'WithValue', 'value')

    def show_value(self) -> str:
        """Returns string representation of value."""
        return str(self._value)


class Client:  # pylint: disable=too-few-public-methods
    """Depends on Service"""

    def __init__(self, service: Service) -> None:
        """Use constructor injection."""
        self._service = service

    def greet_world(self) -> str:
        """Uses Service to greet the world."""
        return self._service.greet("World")


class ServiceFromFactory(NamedTuple):  # pylint: disable=too-few-public-methods
    """Depends on nothing."""
    value: int = 42


@factory()
def service_factory() -> ServiceFromFactory:
    """Create a service."""
    return ServiceFromFactory(40)


class ClientServiceFromFactory(NamedTuple):  # pylint: disable=too-few-public-methods
    """Depends on ServiceFromFactory."""
    service: ServiceFromFactory


class ServiceFromFactoryNonSingleton(NamedTuple):  # pylint: disable=too-few-public-methods
    """Depends on nothing."""
    value: int = 42


@factory(singleton=False)
def service_factory_non_singleton() -> ServiceFromFactoryNonSingleton:
    """Create a service."""
    return ServiceFromFactoryNonSingleton()


class ClientServiceFromFactoryNonSingleton(NamedTuple):  # pylint: disable=too-few-public-methods
    """Depends on ServiceFromFactoryNonSingleton."""
    service: ServiceFromFactoryNonSingleton


def client_func(service: Service) -> str:
    """Use function argument injection."""
    return service.greet("World")


class ClientNonSingleton:  # pylint: disable=too-few-public-methods
    """Depends on ServiceNonSingleton."""

    def __init__(self, service: ServiceNonSingleton) -> None:
        self._service = service


class ClientWithoutTypeAnnotation:  # pylint: disable=too-few-public-methods
    """Depends on some unknown thing."""

    def __init__(self, service) -> None:  # type: ignore
        self._service = service


@component()  # pylint: disable=too-few-public-methods
class Layer3(NamedTuple):
    """Depends on nothing."""
    value: int = 42


@component()  # pylint: disable=too-few-public-methods
class Layer2(NamedTuple):
    """Depends on Layer3"""
    service: Layer3


class Layer1(NamedTuple):  # pylint: disable=too-few-public-methods
    """Depends on Layer2"""
    service: Layer2


@component()  # pylint: disable=too-few-public-methods
class ServiceA(NamedTuple):
    """Depends on nothing."""
    value: str = "A"


@component()  # pylint: disable=too-few-public-methods
class ServiceB(NamedTuple):
    """Depends on nothing."""
    value: str = "B"


class ServiceCNoComponent(NamedTuple):  # pylint: disable=too-few-public-methods
    """Depends on nothing."""
    value: str = "C"


@component()  # pylint: disable=too-few-public-methods
class ClientAB(NamedTuple):
    """Depends on ServiceA and ServiceB."""
    service_a: ServiceA
    service_b: ServiceB


@component()  # pylint: disable=too-few-public-methods
class ClientABDefaultB(NamedTuple):
    """Depends on ServiceA and ServiceB."""
    service_a: ServiceA
    service_b: ServiceB = ServiceB('BDefault')


@component()  # pylint: disable=too-few-public-methods
class ClientACDefaultC(NamedTuple):
    """Depends on ServiceA and ServiceB."""
    service_a: ServiceA
    service_c: ServiceCNoComponent = ServiceCNoComponent('CDefault')


class ClientKWArg:  # pylint: disable=too-few-public-methods
    """Depends on Service"""

    def __init__(self, service: Service, name: str) -> None:
        """Use constructor injection."""
        self._service = service
        self._name = name

    def greet_world(self) -> str:
        """Uses Service to greet the world."""
        return self._service.greet(self._name)


class ClientDependingOnInterface:  # pylint: disable=too-few-public-methods
    """Depends on Service"""

    def __init__(self, service: ServiceInterface) -> None:
        """Use constructor injection."""
        self._service = service

    def greet_world(self) -> str:
        """Uses Service to greet the world."""
        return self._service.greet("World")


class MultiServiceInterface(ABC):  # pylint: disable=too-few-public-methods
    """Define interface for multiple services."""
    _value: str = "Interface"


@component()  # pylint: disable=too-few-public-methods
class ServiceMultiA(MultiServiceInterface):
    """Example service A"""

    def __init__(self) -> None:
        """Depends on nothing."""
        self._value = "A"


@component()  # pylint: disable=too-few-public-methods
class ServiceMultiB(MultiServiceInterface):
    """Example service B"""

    def __init__(self) -> None:
        """Depends on nothing."""
        self._value = "B"


class ClientDependingOnOneOfTwoServices:  # pylint: disable=too-few-public-methods
    """Depends on ServiceMultiA or ServiceMultiB"""

    def __init__(self, service: MultiServiceInterface) -> None:
        """Use constructor injection."""
        self._service = service


class ClientDependingOnAllMultiServiceInterfaceImpls:  # pylint: disable=too-few-public-methods
    """Depends on ServiceMultiA and ServiceMultiB"""

    def __init__(self, services: List[MultiServiceInterface]) -> None:
        """Use constructor injection."""
        self._services = services


class BasicTest(unittest.TestCase):
    """Check basic functionality."""

    def test_assemble(self) -> None:
        """Basic component lookup."""
        self.assertEqual("Hello, World!", assemble(Client).greet_world())

    def test_assemble_func(self) -> None:
        """Free function instead of constructor."""
        self.assertEqual("Hello, World!", assemble(client_func))

    def test_singleton(self) -> None:
        """Multiple calls to assemble shall return the same object."""
        self.assertTrue(assemble(Client)._service is assemble(Client)._service)  # pylint: disable=protected-access

    def test_non_singleton(self) -> None:
        """Multiple calls to assemble shall return the same object."""
        self.assertTrue(
            assemble(ClientNonSingleton)._service is not  # pylint: disable=protected-access
            assemble(ClientNonSingleton)._service)  # pylint: disable=protected-access

    def test_namedtuple(self) -> None:
        """Nested injection."""
        self.assertEqual(42, assemble(Layer2).service.value)

    def test_multiple_layers(self) -> None:
        """Nested injection."""
        self.assertEqual(42, assemble(Layer1).service.service.value)


class FactoryTest(unittest.TestCase):
    """Check factory functionality."""

    def test_construct_service(self) -> None:
        """Factory function creates service."""
        self.assertEqual(40, assemble(ServiceFromFactory).value)

    def test_factory(self) -> None:
        """Factory function produces dependency."""
        self.assertEqual(40, assemble(ClientServiceFromFactory).service.value)

    def test_factory_singleton(self) -> None:
        """Factory function as component."""
        self.assertTrue(assemble(ClientServiceFromFactory).service is
                        assemble(ClientServiceFromFactory).service)  # pylint: disable=protected-access

    def test_factory_non_singleton(self) -> None:
        """Factory function as component."""
        self.assertTrue(
            assemble(ClientServiceFromFactoryNonSingleton).service is not  # pylint: disable=protected-access
            assemble(ClientServiceFromFactoryNonSingleton).service)  # pylint: disable=protected-access


class ValueTest(unittest.TestCase):
    """Check value store."""

    def test_value(self) -> None:
        """Using value from configuration."""
        config = configparser.ConfigParser()
        config.read_string("""
            [WithValue]
            value = 42
        """)
        set_values_from_config(config)
        self.assertEqual('42', WithValue().show_value())

    def test_add_config_value(self) -> None:
        """Manually adding a value."""
        set_values({})
        add_values({'WithValue': {'value': 43}})
        self.assertEqual('43', WithValue().show_value())

    def test_try_replace_value(self) -> None:
        """Values must be unique."""
        set_values({})
        add_values({'WithValue': {'value': 43}})
        with self.assertRaises(ValueError):
            add_values({'WithValue': {'value': 43}})


class ErrorTest(unittest.TestCase):
    """Check exceptions."""

    def test_unknown_service_type(self) -> None:
        """A service parameter needs a type annotation."""
        with self.assertRaises(TypeError):
            assemble(ClientWithoutTypeAnnotation)

    def test_double_registration(self) -> None:
        """A class may only be registered once."""
        with self.assertRaises(TypeError):
            @component()  # pylint: disable=too-few-public-methods,unused-variable
            @component()
            class Duplicate:
                """Class to be registered multiple times."""

    def test_ambiguous(self) -> None:
        """Ambiguous dependency."""
        with self.assertRaises(TypeError):
            assemble(ClientDependingOnOneOfTwoServices)

    def test_additional_factory(self) -> None:
        """Ambiguous dependency due to a factory."""
        with self.assertRaises(TypeError):
            @component()
            def service_factory_forbidden() -> Service:  # pylint: disable=unused-variable
                """Conflict with component."""
                return Service()


class AbstractTest(unittest.TestCase):
    """Check interfaces."""

    def test_interface(self) -> None:
        """Concrete object shall be injected."""
        self.assertEqual("Hello, World!",
                         assemble(ClientDependingOnInterface).greet_world())

    def test_multiple_services(self) -> None:
        """Multi-injection."""
        client = assemble(ClientAB)
        self.assertEqual('A', client.service_a.value)
        self.assertEqual('B', client.service_b.value)

    def test_manual_overwrite(self) -> None:
        """Prefer manually provided services."""
        client = assemble(ClientAB, service_b=ServiceB('BManual'))
        self.assertEqual('A', client.service_a.value)
        self.assertEqual('BManual', client.service_b.value)

    def test_default_arguments(self) -> None:
        """Use default arguments in clients if nothing else is given."""
        client = assemble(ClientACDefaultC)
        self.assertEqual('A', client.service_a.value)
        self.assertEqual('CDefault', client.service_c.value)

    def test_overwrite_default(self) -> None:
        """Prefer components over default arguments."""
        client = assemble(ClientABDefaultB)
        self.assertEqual('A', client.service_a.value)
        self.assertEqual('B', client.service_b.value)

    def test_service_list(self) -> None:
        """Inject multiple services as List."""
        client = assemble(ClientDependingOnAllMultiServiceInterfaceImpls)
        self.assertEqual(2, len(client._services))  # pylint: disable=protected-access
        self.assertEqual("A", client._services[0]._value)  # pylint: disable=protected-access
        self.assertEqual("B", client._services[1]._value)  # pylint: disable=protected-access


@component(profiles=['prod'])  # pylint: disable=too-few-public-methods
class ServiceProd(ServiceInterface):
    """Example service"""

    def greet(self, name: str) -> str:
        """Returns greeting message."""
        return f'prod: Hello, {name}!'


@component(profiles=['test', 'dev'])  # pylint: disable=too-few-public-methods
class ServiceTest(ServiceInterface):
    """Example service"""

    def greet(self, name: str) -> str:
        """Returns greeting message."""
        return f'testdev: Hello, {name}!'


class ProfileTest(unittest.TestCase):
    """Check profiles."""

    def test_no_profile_ok(self) -> None:
        """Object shall be injected."""
        self.assertEqual("Hello, World!",
                         assemble(ClientDependingOnInterface).greet_world())

    def test_selected_profile_1(self) -> None:
        """Object is available."""
        self.assertEqual("prod: Hello, World!",
                         assemble(ClientDependingOnInterface, 'prod').greet_world())

    def test_selected_profile_2(self) -> None:
        """Object is available."""
        self.assertEqual("testdev: Hello, World!",
                         assemble(ClientDependingOnInterface, 'test').greet_world())

    def test_selected_profile_3(self) -> None:
        """Object is available."""
        self.assertEqual("testdev: Hello, World!",
                         assemble(ClientDependingOnInterface, 'dev').greet_world())

    def test_no_profile_fail(self) -> None:
        """Object is not available."""
        with self.assertRaises(TypeError):
            assemble(ClientDependingOnInterface, "unknown_profile")



"""
enterprython - Type-based dependency injection
"""

import configparser
import inspect
import sys
from typing import Any, Callable, Dict, Optional, Type, TypeVar, List, Generic, Union, Tuple

VER_3_7_AND_UP = sys.version_info[:3] >= (3, 7, 0)  # PEP 560

# pylint: disable=no-name-in-module
if VER_3_7_AND_UP:
    from typing import _GenericAlias  # type: ignore
else:
    pass
# pylint: enable=no-name-in-module

TypeT = TypeVar('TypeT')


class _Component(Generic[TypeT]):  # pylint: disable=unsubscriptable-object
    """Internal class to store components for DI."""

    def __init__(self, the_type: Callable[..., TypeT],
                 target_types: Tuple[Type[Any], ...],
                 singleton: bool,
                 profiles: List[str]) -> None:
        """Figure out and store base classes."""
        self._type = the_type
        self._is_singleton = singleton
        self._profiles = profiles
        self._target_types = target_types
        self._instance: Optional[TypeT] = None

    def matches(self, the_type: Callable[..., TypeT],
                profile: Optional[str]) -> bool:
        """Check if component can be used for injection of the_type."""
        if self._profiles and not profile:
            return False
        if not self._profiles and profile:
            return False
        if profile and profile not in self._profiles:
            return False
        return the_type in self._target_types

    def set_instance_if_singleton(self, instance: TypeT) -> None:
        """If this component is a singleton, set its instance."""
        if self._is_singleton:
            assert self._instance is None
            self._instance = instance

    def get_instance(self) -> Optional[TypeT]:
        """Access singleton instance if present."""
        assert self._is_singleton or self._instance is None
        return self._instance

    def get_type(self) -> Callable[..., TypeT]:
        """Underlying target type of component."""
        return self._type


class _Factory(Generic[TypeT]):  # pylint: disable=unsubscriptable-object
    """Internal class to store factories for DI."""

    def __init__(self, func: Callable[..., TypeT],
                 singleton: bool,
                 profiles: List[str]) -> None:
        """Figure out and store base classes."""
        return_type = inspect.signature(func).return_annotation
        if return_type is inspect.Signature.empty:
            raise TypeError(f'Unknown return type of {func.__name__}')
        self._func = func
        self._is_singleton = singleton
        self._profiles = profiles
        self._return_type: Type[Any] = return_type
        self._target_types: Tuple[Type[Any], ...] = inspect.getmro(return_type)
        self._instance: Optional[TypeT] = None

    def matches(self, the_type: Callable[..., TypeT],
                profile: Optional[str]) -> bool:
        """Check if factory can be used for injection of the_type."""
        if self._profiles and not profile:
            return False
        if not self._profiles and profile:
            return False
        if profile and profile not in self._profiles:
            return False
        return the_type in self._target_types

    def get_instance(self) -> Optional[TypeT]:
        """Access singleton instance if present."""
        if not self._is_singleton or self._instance is None:
            self._instance = self._func()
        return self._instance

    def get_return_type(self) -> Type[Any]:
        """Type of constructed object."""
        return self._return_type


ValueType = Union[str, float, int, bool]

ENTERPRYTHON_VALUES: Dict[str, Dict[str, ValueType]] = {}
ENTERPRYTHON_COMPONENTS: List[_Component[Any]] = []
ENTERPRYTHON_FACTORIES: List[_Factory[Any]] = []


def add_values(new_values: Dict[str, Dict[str, ValueType]]) -> None:
    """Extent current value store (section, name, value)."""
    global ENTERPRYTHON_VALUES  # pylint: disable=global-statement
    for section, names_with_values in new_values.items():
        if section in ENTERPRYTHON_VALUES:
            for name, new_value in names_with_values.items():
                if name in ENTERPRYTHON_VALUES[section]:
                    raise ValueError(f'Duplicate value: {section}.{name}')
    for section, names_with_values in new_values.items():
        for name, new_value in names_with_values.items():
            if section not in ENTERPRYTHON_VALUES:
                ENTERPRYTHON_VALUES[section] = {}
            ENTERPRYTHON_VALUES[section][name] = new_value


def add_value(section: str, name: str, new_value: ValueType) -> None:
    """Add a single new value to the store."""
    add_values({section: {name: new_value}})


def set_values(values: Dict[str, Dict[str, ValueType]]) -> None:
    """Set global enterprython value store (section, name, value)."""
    global ENTERPRYTHON_VALUES  # pylint: disable=global-statement
    ENTERPRYTHON_VALUES = {}
    add_values(values)


def set_values_from_config(config: configparser.ConfigParser) -> None:
    """Set global enterprython value store from a ConfigParser."""
    set_values({s: dict(config.items(s)) for s in config.sections()})


def _create(the_type: Callable[..., TypeT],
            profile: Optional[str] = None) -> Optional[TypeT]:
    stored_factory = _get_factory(the_type, profile)
    if stored_factory:
        return stored_factory.get_instance()

    stored_component = _get_component(the_type, profile)
    if stored_component:
        instance = stored_component.get_instance()
        if instance is not None:
            return instance
    return None


def _get_parameters(signature: inspect.Signature) -> Dict[str, Type[Any]]:
    parameters: Dict[str, Type[Any]] = {}
    for param in signature.parameters.values():
        if param.name == 'self':
            continue
        if param.kind != inspect.Parameter.POSITIONAL_OR_KEYWORD:
            raise TypeError('Only parameters of kind POSITIONAL_OR_KEYWORD '
                            'supported in target functions.')
        if param.annotation is inspect.Signature.empty:
            raise TypeError('Parameter needs needs a type annotation.')
        parameters[param.name] = param.annotation
    return parameters


def assemble(the_type: Callable[..., TypeT],
             profile: Optional[str] = None,
             **kwargs: Any) -> TypeT:
    """Create an instance of a certain type,
    using constructor injection if needed."""

    global ENTERPRYTHON_COMPONENTS  # pylint: disable=global-statement

    ready_result = _create(the_type, profile)
    if ready_result is not None:
        return ready_result

    signature = inspect.signature(the_type)

    parameters = _get_parameters(signature)

    arguments: Dict[str, Any] = kwargs
    uses_manual_args = False
    for parameter_name, parameter_type in parameters.items():
        if parameter_name in arguments:
            uses_manual_args = True
            continue
        if _is_list_type(parameter_type):
            parameter_components = _get_components(
                _get_list_type_elem_type(parameter_type), profile)
            arguments[parameter_name] = list(map(assemble,
                                                 map(lambda comp: comp.get_type(),
                                                     parameter_components)))
        else:
            parameter_component = _get_component(parameter_type, profile)
            param_factory = _get_factory(parameter_type, profile)
            if parameter_component is not None:
                arguments[parameter_name] = assemble(
                    parameter_component.get_type())  # parameter_type?
            elif param_factory:
                arguments[parameter_name] = param_factory.get_instance()
    result = the_type(**arguments)
    stored_component = _get_component(the_type, profile)
    if stored_component and not uses_manual_args:
        stored_component.set_instance_if_singleton(result)
    return result


def value(the_type: Callable[..., TypeT], config_section: str, value_name: str) -> TypeT:
    """Get a config value from the global enterprython config store."""
    assert the_type in [bool, float, int, str]
    return the_type(ENTERPRYTHON_VALUES[config_section][value_name])


def component(singleton: bool = True,  # pylint: disable=dangerous-default-value
              profiles: List[str] = []) -> Callable[[Callable[..., TypeT]],  # pylint: disable=dangerous-default-value
                                                    Callable[..., TypeT]]:
    """Annotation to register a class to be available for DI."""

    def register(the_class: Callable[..., TypeT]) -> Callable[..., TypeT]:
        """Register component and forward type."""
        if not inspect.isclass(the_class):
            raise TypeError('Only classes can be registered as components.')
        if inspect.isabstract(the_class):
            raise TypeError(f'Can not register abstract class as component.')
        global ENTERPRYTHON_COMPONENTS  # pylint: disable=global-statement
        target_types = inspect.getmro(the_class)  # type: ignore
        _add_component(the_class, target_types, singleton, profiles)
        return the_class

    return register


def factory(singleton: bool = True,  # pylint: disable=dangerous-default-value
            profiles: List[str] = []) -> Callable[[Callable[..., TypeT]],
                                                  Callable[..., TypeT]]:
    """Annotation to register a factory to be available for DI."""

    def register(func: Callable[..., TypeT]) -> Callable[..., TypeT]:
        """Register component and forward type."""
        if not inspect.isfunction(func):
            raise TypeError('Only functions can be registered as factories.')
        global ENTERPRYTHON_COMPONENTS  # pylint: disable=global-statement
        _add_factory(func, singleton, profiles)
        return func

    return register


def _get_components(the_type: Callable[..., TypeT],
                    profile: Optional[str]) -> List[_Component[TypeT]]:
    """Return stored component for type if available."""
    return [stored_component
            for stored_component in ENTERPRYTHON_COMPONENTS
            if stored_component.matches(the_type, profile)]


def _get_component(the_type: Callable[..., TypeT],
                   profile: Optional[str]) -> Optional[_Component[TypeT]]:
    """Return stored component for type if available."""
    components = _get_components(the_type, profile)
    if len(components) > 1:
        raise TypeError(f'Ambiguous dependency {the_type.__name__}.')
    if not components:
        return None
    return components[0]


def _get_factories(the_type: Callable[..., TypeT],
                   profile: Optional[str]) -> List[_Factory[TypeT]]:
    """Return stored factories for type if available."""
    return [stored_factory
            for stored_factory in ENTERPRYTHON_FACTORIES
            if stored_factory.matches(the_type, profile)]


def _get_factory(the_type: Callable[..., TypeT],
                 profile: Optional[str]) -> Optional[_Factory[TypeT]]:
    """Return stored factory for type if available."""
    factories = _get_factories(the_type, profile)
    if len(factories) > 1:
        raise TypeError(f'Multiple factories available for {the_type.__name__}.')
    if not factories:
        return None
    return factories[0]


def _add_component(the_type: Callable[..., TypeT],
                   target_types: Tuple[Type[Any], ...],
                   singleton: bool,
                   profiles: List[str]) -> None:
    """Store new component for DI."""
    global ENTERPRYTHON_COMPONENTS  # pylint: disable=global-statement
    new_component = _Component(the_type, target_types, singleton, profiles)
    if _get_component(new_component.get_type(), None) is not None:
        raise TypeError(f'{the_type.__name__} '
                        'already registered as component.')

    for profile in profiles:
        if _get_component(new_component.get_type(), profile) is not None:
            raise TypeError(f'{the_type.__name__} '
                            'already registered as component for profile "{profile}".')
    ENTERPRYTHON_COMPONENTS.append(new_component)


def _add_factory(func: Callable[..., TypeT],
                 singleton: bool,
                 profiles: List[str]) -> None:
    """Store new factory for DI."""
    global ENTERPRYTHON_FACTORIES  # pylint: disable=global-statement
    new_factory = _Factory(func, singleton, profiles)
    if _get_factory(new_factory.get_return_type(), None) is not None:
        raise TypeError(f'{func.__name__} '
                        'already registered as component.')

    for profile in profiles:
        if _get_factory(new_factory.get_return_type(), profile) is not None:
            raise TypeError(f'{func.__name__} '
                            'already registered as component for profile "{profile}".')
    ENTERPRYTHON_FACTORIES.append(new_factory)


def _is_list_type(the_type: Callable[..., TypeT]) -> bool:
    try:
        if VER_3_7_AND_UP:
            return _is_instance(the_type,
                                _GenericAlias) and _type_origin_is(the_type, list)
        return issubclass(the_type, List)  # type: ignore
    except TypeError:
        return False


def _is_instance(the_value: TypeT, the_type: Callable[..., TypeT]) -> bool:
    return isinstance(the_value, the_type)  # type: ignore


def _type_origin_is(the_type: Callable[..., TypeT], origin: Any) -> bool:
    assert hasattr(the_type, '__origin__')
    return the_type.__origin__ is origin  # type: ignore


def _get_list_type_elem_type(list_type: Callable[..., TypeT]) -> Callable[..., Any]:
    """Return the type of a single element of the list type."""
    assert _is_list_type(list_type)
    list_args = list_type.__args__  # type: ignore
    assert len(list_args) == 1
    return list_args[0]  # type: ignore



import pypandoc
import setuptools

with open("README.md", "r") as fh:
    long_description = pypandoc.convert('README.md', 'rst')

setuptools.setup(
    name="enterprython",
    version="0.5.3",
    author="Tobias Hermann",
    author_email="editgym@gmail.com",
    description="Type-based dependency injection",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="http://github.com/Dobiasd/enterprython",
    packages=setuptools.find_packages(),
    classifiers=(
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)



from ._inject import add_value, add_values, set_values, set_values_from_config
from ._inject import assemble, component, factory, value

name = "enterprython"

__author__ = "Tobias Hermann"
__copyright__ = "Copyright 2018, Tobias Hermann"
__email__ = "editgym@gmail.com"
__license__ = "MIT"
